/* Beeper - makes short, high pitched beeps */

#include "beeper.h"

static const int error_exit_status = -1; 

/* Inline assembler for direct access */

#define inline __inline__

__inline__ static void outb(char value, unsigned short port)
{
  __asm__ volatile ("outb %0,%1"
                    ::"a" ((char) value), "d"((unsigned short) port));
}

__inline__ static char inb(unsigned short port)
{
  char _v;
  __asm__ volatile ("inb %1,%0"
                    :"=a" (_v):"d"((unsigned short) port));

  return _v;
}

__inline__ static void cli()
{
  __asm__ volatile ("cli");
}

__inline__ static void sti()
{
  __asm__ volatile ("sti");
}

/* Thread: Wait for signal, then beep with set frequency */

void * beep_me_thread(struct beeper *beep) 
{
 int frequency;
 int length;
 int div;
 
 /* Loop forever*/
 while (1) {
 
  /* Lock shared variables */ 
  pthread_mutex_lock(&beep->sound_mutex);
  /* Wait for signal */
  pthread_cond_wait(&beep->sound_cond,&beep->sound_mutex);
  /* Get the frequency */
  frequency=beep->frequency;
  length=beep->length;
  /* Unlock state */
  pthread_mutex_unlock(&beep->sound_mutex);

  /* Beep now */
  /* On */
  div=frequency ? 4770000/4/frequency : 0;
  iopl(3);
  cli();
  outb(0xb6, 0x43);
  outb(div & 0xff, 0x42);
  outb(div >> 8, 0x42);
  outb(inb(0x61) | 0x03, 0x61);
  sti();
  iopl(0);
  /* Pause */
  usleep(length*1000);
  /* Off */
  iopl(3);
  cli();
  outb(inb(0x61) & 0xfc,0x61);
  sti();
  iopl(0);
 
 }
}

/* Initialize the beeper */

int beep_init(struct beeper *beep)
{
  /* Get permissions */
  if (ioperm(0x61,1,1)) {
   perror("ioperm(0x61)");
   return(-1);
  }
  if (ioperm(0x42,2,1)) {
   ioperm(0x61,1,0);
   perror("ioperm(0x42)");
   return(-1);
  }
 
 /* Initialize sycro mutex */
 if (pthread_mutex_init(&beep->sound_mutex, NULL)==-1) {
  perror("pthread_mutex_init()");
  return(-1);
 }

 /* Initialize sycro conditional */
 if (pthread_cond_init(&beep->sound_cond, NULL)==-1) {
  perror("pthread_cond_init()");
  return(-1);
 } 
        
 /* Start new thread */
 if (pthread_create(&beep->sound_thread, NULL,(void *)beep_me_thread,(void *)beep)==-1) {
  perror ("pthread_create()");
  return(-1);
 }
  
 return(0);
}


int beep_me (struct beeper *beep, int frequency, int length)
{
 /* Lock state */
 pthread_mutex_lock(&beep->sound_mutex);
 /* Store frequency and duration */
 beep->frequency=frequency;
 beep->length=length;
 /* Lock state */
 pthread_mutex_unlock(&beep->sound_mutex);
 
 /* Signal to beeper thread to beep */ 
 pthread_cond_signal(&beep->sound_cond);
  
 return(0); 
}

/* Close beeper */

int beep_close(struct beeper *beep)
{
 /* Off */
 iopl(3);
 cli();
 outb(inb(0x61) & 0xfc,0x61);
 sti();
 iopl(0);
 
 /* Release IO-ports */
 ioperm(0x61,1,0);
 ioperm(0x42,2,0);
   
 return(0);
}
