/* Sounder */

/* Program waveform externally after Init */

#include "sounder.h"

static const int error_exit_status = -1; 

/* Thread: Wait for signal, then play audio */

void *sound_me_thread(struct sounder *sound) 
{
 /* Loop forever */
 while (1==1) {
  /* Wait for signal */
  pthread_cond_wait(&sound->sound_cond,&sound->sound_mutex);

#ifdef DEBUG
  fprintf (stderr,"Playing sample\n");
#endif
   
  if (sound->do_reset>0) { 
   /* Reset sound now */
   if (write(sound->audio_fd, sound->audio_buffer_reset, sound->audio_buffer_size)==-1) {
    perror ("write()");
   }
   sound->do_reset--;
   fprintf(stderr,"Reset sequence ...\n");
  } else { 
   /* Normal sound now */
   if (write(sound->audio_fd, sound->audio_buffer, sound->audio_buffer_size)==-1) {
    perror ("write()");
   }
  }                                                                                                                                               
 }
}

/* Initialize the sounder */

int sound_init(struct sounder *sound, char *devicename, int size)
{
 int frag;
 int format;
 int stereo;
 int speed;
 
 /* No reset */
 sound->do_reset=0;
 
 /* Open sound device */
 if ((sound->audio_fd=open(devicename,O_WRONLY,0)) == -1) {
  perror(devicename);
  return(-1);
 }

 /* Shorten fragment size */
 frag = 0x00030009;
 if (ioctl(sound->audio_fd, SNDCTL_DSP_SETFRAGMENT, &frag)==-1) {
  perror("ioctl()");
  exit(-1);
 }
                               
 /* Setup sound device */
 /* Bits */
 format=AFMT_S16_LE;
 if (ioctl(sound->audio_fd,SNDCTL_DSP_SETFMT,&format)==-1) {
  perror("ioctl()");
  exit(-1);
 }
 if (format != AFMT_S16_LE) {
  perror("Audioformat AFMT_S16_BE not suppoted");
  exit(-1);
 }
 /* Channels */
 stereo=0;
 if (ioctl(sound->audio_fd,SNDCTL_DSP_STEREO,&stereo)==-1) {
  perror("ioctl()");
  exit(-1);
 }
 if (stereo != 0) {
  perror("MONO not suppoted");
  exit(-1);
 }
 /* Speed */
 speed=44100;
 if (ioctl(sound->audio_fd,SNDCTL_DSP_SPEED,&speed)==-1) {
  perror("ioctl()");
  exit(-1);
 }
 if (speed != 44100) {
  perror("44100Hz playback not suppoted");
  exit(-1);
 }
    
 /* Initialize sycro mutex */
 if (pthread_mutex_init(&sound->sound_mutex, NULL)==-1) {
  perror("pthread_mutex_init()");
  return(-1);
 }

 /* Initialize sycro conditional */
 if (pthread_cond_init(&sound->sound_cond, NULL)==-1) {
  perror("pthread_cond_init()");
  return(-1);
 } 
 
 /* Create audio buffer */
 sound->audio_buffer=(unsigned char *)malloc(size);
 if (sound->audio_buffer==NULL) {
  perror("malloc");
  return(-1);
 }
 sound->audio_buffer_size=size;

 /* Create reset audio buffer */
 sound->audio_buffer_reset=(unsigned char *)malloc(size);
 if (sound->audio_buffer_reset==NULL) {
  perror("malloc");
  return(-1);
 }
          
 /* Start new thread */
 if (pthread_create(&sound->sound_thread, NULL,(void *)sound_me_thread,(void *)sound)==-1) {
  perror ("pthread_create()");
  return(-1);
 }
  
 return(0);
}

/* Initialize mixer */

int mixer_init(char *devicename, int volume)
{
 int mix_fd;
 int vol;
 
 /* Open mixer device */
 if ((mix_fd=open(devicename,O_WRONLY,0)) == -1) {
  perror(devicename);
  return(-1);
 }

 /* Adjust for stereo volume controls */
 vol=(volume + (volume <<8));

 /* Set main volume value */
 if (ioctl(mix_fd, SOUND_MIXER_WRITE_VOLUME, &vol) == -1) {
  perror ("ioctl(SOUND_MIXER_WRITE)");
  return(-1);
 }

 /* Set PCM volume value */
 if (ioctl(mix_fd, SOUND_MIXER_WRITE_PCM, &vol) == -1) {
  perror ("ioctl(SOUND_MIXER_WRITE)");
  return(-1);
 }
 
 return (0);
}

/* Signal thread to play */

int sound_me (struct sounder *sound)
{
 /* Signal to beeper thread to beep */ 
 pthread_cond_signal(&sound->sound_cond);
  
 return(0); 
}



/* Close sounder */

int sound_close(struct sounder *sound)
{
 /* Close device */
 close (sound->audio_fd);
 
 /* Free memory */
 free(sound->audio_buffer);
     
 return(0);
}
